/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <QtGui/qmenu.h>

#include "KTASysTray.h"

KTASysTray::KTASysTray( QWidget* parent,  const char* name ) : KSystemTrayIcon( parent ) {
    this->interfaceMenu = new QMenu( "Interface" );
    this->contextMenu()->addMenu( this->interfaceMenu );

    // Reset interface name
    this->setInterfaceName( "" );

    // Load icon
    this->setIcon( KIconLoader::global()->loadIcon( "network-workgroup", KIconLoader::Panel ) );
    // Update tooltip
    this->updateTraffic( "0.0B", "0.0B" );
}

KTASysTray::~KTASysTray() {
}

//Update interface name
void KTASysTray::setInterfaceName( QString interfaceName ) {
	this->interfaceName = interfaceName;
}

//Called to update the tooltip
void KTASysTray::updateTraffic( QString downTraffic, QString upTraffic ) {
	//QToolTip::add( this, QRect(0,0,200,20), "KTrafficAnalyzer (" + this->interfaceName + ")\nDown: " + downTraffic + "\nUp: " + upTraffic );
	setToolTip( "KTrafficAnalyzer (" + this->interfaceName + ")\n" + i18n("Down") + ": " + downTraffic + "\n" + i18n( "Up" ) + ": " + upTraffic );
}

// Called when the quick selection menu for the interfaces should be updated
void KTASysTray::updateInterfaces( QStringList interfaces, QString currInterface ) {
    this->interfaceMenu->clear();

    QActionGroup *interfaceActions = new QActionGroup( this );

    for( int i = 0; i < interfaces.size(); i++ ) {
        QAction *interfaceAction = this->interfaceMenu->addAction( interfaces.at(i) );
        interfaceAction->setCheckable( true );

        if( interfaces.at(i).compare( currInterface ) == 0 ) {
            interfaceAction->setChecked(true);
        }

        interfaceActions->addAction(interfaceAction);

        QObject::connect( interfaceActions, SIGNAL(triggered(QAction*)), this, SLOT(interfaceEntryTriggered(QAction*)) );

        //QObject::connect( interfaceAction, SIGNAL(changed()), this, SLOT(interfaceEntryChanged()) );
    }
}

void KTASysTray::interfaceEntryTriggered(QAction *source) {
    emit interfaceChanged(source->text().replace('&',"",Qt::CaseInsensitive));
}
