/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "SSHTrafficInterface.h"

SSHTrafficInterface::SSHTrafficInterface() {
//	printf( "Constructor of SSHTrafficInterface!\n" );

	TrafficInterface::TrafficInterface();
	
	//Get SSH-Info
	this->sshHost = KTASettings::sSHHost();
	
	this->sshConsole = new remoteRead();
	*this->sshConsole << "ssh";
	*this->sshConsole << KTASettings::sSHHost() << "-l" << KTASettings::sSHUser();
//	*this->sshConsole << "root@88.198.33.21" << "-p" << "54763";
	//Now add any additional parameters
	QStringList addParas = QStringList::split( " ", KTASettings::sSHParameter() );
	for ( QStringList::Iterator it = addParas.begin(); it != addParas.end(); ++it ) {
		*this->sshConsole << *it;
	}
	
	//Enable / Disable Debug
	this->sshConsole->setDebug( KTASettings::sSHDebug() );

	//Start SSH-Connect and login into remote host
	this->sshConsole->setUsePty( KProcess::Communication(KProcess::All + KProcess::MergedStderr), false);
	this->sshConsole->start( KProcess::DontCare );
	this->sshConsole->login( (char * ) KTASettings::sSHPass().ascii() );
}

SSHTrafficInterface::~SSHTrafficInterface() {
//	printf( "SSHTrafficInterface destructor!\n" );
	//Close connection and delete handler
	this->sshConsole->logout();
	this->sshConsole->closeAll();
	delete this->sshConsole;
}

//Read traffic differences for interface interfaceName
//Error codes:
// < 1 - no error
// 1 - /proc/net/dev couldn't be read
// 2 - Interface doesn't exist
int SSHTrafficInterface::readTrafficDiff( char *currInterfaceName ) {
	QString data;					//Data buffer for file content

	//Set interface name
	this->interfaceName = currInterfaceName;

	//Check if we have a different interface now (if yes reset values)
	if( QString::compare( currInterfaceName, this->lastInterfaceName ) != 0 ) {
		this->downloadDiff = 0.0;
		this->uploadDiff = 0.0;
		this->oldDownload = 0.0;
		this->oldUpload = 0.0;
		this->lastInterfaceName = currInterfaceName;
		this->interfaceChanged = true;
	}
	else {
		this->interfaceChanged = false;
	}

	//Create command line
	QString command( "cat /proc/net/dev | grep " );
	command.append( this->interfaceName );
	
	//Send command to remote host
	this->sshConsole->writePtyLine( (char*) command.ascii() );
	
	//Read return stuff
	char buffer[4096];
	this->sshConsole->readPtyUntil( ":~ # ", buffer );

	//Split up buffer into lines
	QStringList bufferLines = QStringList::split( "\n", buffer );
	//The seoncd line should be our data
	QString trafficInfo = bufferLines[1];
	
//	printf( "Traffic-Info: %s\n", trafficInfo.ascii() );
	
	//eth0: 5299187   15784    0    0    0     0          0         0  1139753   14261    0    0    0     0       0          0
	//Apply regular expression at our line
	QRegExp netInfo( "\\s*(\\S+):\\s*(\\d+)\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s*(\\d+)" );
	//Check if search was succesfull
	float trafficDownload;
	float trafficUpload;
	//Apply regular expression
	netInfo.search( trafficInfo ) ;

	//Get traffic values
	trafficDownload = netInfo.cap( 2 ).toFloat();
	trafficUpload = netInfo.cap( 3 ).toFloat();
	
	//Check if first run
	if( this->oldDownload == 0.0 || this->oldUpload == 0.0 ) {
		this->oldDownload = trafficDownload;
		this->oldUpload = trafficUpload;
	}
	
	//Calculate differences
	this->downloadDiff = trafficDownload - this->oldDownload;
	this->uploadDiff = trafficUpload - this->oldUpload;
	
	//Check if interface was up all time (else we might get a negative difference)
	if( this->downloadDiff < 0 || this->uploadDiff < 0 ) {
		this->downloadDiff = 0;
		this->uploadDiff = 0;
	}
	
	//Save new values
	this->oldDownload = trafficDownload;
	this->oldUpload = trafficUpload;
			
	return 0;
}

//Get a list of all available interfaces
QStringList SSHTrafficInterface::getInterfaces() {
	//QStringList start
	QStringList interfaces;

	//Read whole /proc/net/dev file
	char fileBuffer[4096];
	this->sshConsole->writePtyLine( "cat /proc/net/dev" );
	this->sshConsole->readPtyUntil( ":~ # ", fileBuffer );
	
	//Now split file content into lines
	QStringList fileBufferLines = QStringList::split( "\n", fileBuffer );
	
	//Now go through all lines and check for interfaces
	int i = 0;
	for( i = 0; i < fileBufferLines.count(); i++ ) {
       	QRegExp interfaceInfo( "^\\s*(\\S+):\\s*\\d+.*" );
		if( interfaceInfo.search( fileBufferLines[i] ) != -1 ) {
			interfaces << interfaceInfo.cap( 1 );
		}
    }
    //Return interface list
    return interfaces;
}

//Return the interface name
QString SSHTrafficInterface::getInterfaceName() {
	return (this->sshHost + "_" + this->interfaceName);
}

//Update Debug-Settings
void SSHTrafficInterface::setDebug( bool bDebug ) {
	this->sshConsole->setDebug( bDebug );
}
