/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "SYSTrafficInterface.h"

SYSTrafficInterface::SYSTrafficInterface() {
//	printf( "Constructor of SYSTrafficInterface!\n" );

	TrafficInterface();
}

//Read traffic differences for interface interfaceName
//Error codes:
// < 1 - no error
// 1 - Statistics file couldn't be read
// 2 - Interface doesn't exist
int SYSTrafficInterface::readTrafficDiff( char *currInterfaceName ) {
	QString data;					//Data buffer for file content

	//Set interface name
	this->interfaceName = currInterfaceName;

	//Check if we have a different interface now (if yes reset values)
	if( QString::compare( currInterfaceName, this->lastInterfaceName ) != 0 ) {
		this->downloadDiff = 0.0;
		this->uploadDiff = 0.0;
		this->oldDownload = 0.0;
		this->oldUpload = 0.0;
		this->lastInterfaceName = currInterfaceName;
		this->interfaceChanged = true;
	}
	else {
		this->interfaceChanged = false;
	}

	//Create base path
	QString basePath( "/sys/class/net/" );
	basePath.append( currInterfaceName );
	//Check if that path (interface) exists
	QDir *existCheck = new QDir();
	if( !existCheck->exists( basePath ) ) return 2;

	//Traffic vars
	float trafficDownload;
	float trafficUpload;

	//Get file for rx_bytes
	QFile *sysInfo = new QFile( basePath + "/statistics/rx_bytes" );
	sysInfo->open( QIODevice::ReadOnly );
	//Check if file can be opened
	if( !sysInfo->isOpen() ) {
		//...if not return error
		return 1;
	}
	//Read out rx_bytes
	data = sysInfo->readLine();
	trafficDownload = data.toFloat();
	sysInfo->close();

	//Get file for tx_bytes
	sysInfo->setFileName( basePath + "/statistics/tx_bytes" );
	sysInfo->open( QIODevice::ReadOnly );
	//Check if file can be opened
	if( !sysInfo->isOpen() ) {
		//...if not return error
		return 1;
	}
	//Read out tx_bytes
	data = sysInfo->readLine();
	trafficUpload = data.toFloat();
	sysInfo->close();


	//Check if first run
	if( this->oldDownload == 0.0 || this->oldUpload == 0.0 ) {
		this->oldDownload = trafficDownload;
		this->oldUpload = trafficUpload;
	}

	//Calculate differences
	this->downloadDiff = trafficDownload - this->oldDownload;
	this->uploadDiff = trafficUpload - this->oldUpload;

	//Check if interface was up all time (else we might get a negative difference)
	if( this->downloadDiff < 0 || this->uploadDiff < 0 ) {
		this->downloadDiff = 0;
		this->uploadDiff = 0;
	}

	//Save new values
	this->oldDownload = trafficDownload;
	this->oldUpload = trafficUpload;

	return 0;
}

//Get a list of all available interfaces
QStringList SYSTrafficInterface::getInterfaces() {
	//QStringList start
	QStringList interfaces;

	//Create QDir object
	QDir *dirList = new QDir( "/sys/class/net/" );
	interfaces = dirList->entryList( QDir::Dirs );
	interfaces.removeAt( interfaces.indexOf( "." ) );
	interfaces.removeAt( interfaces.indexOf( ".." ) );

    //Return interface list
    return interfaces;
}
