/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "TrafficInterface.h"

TrafficInterface *TrafficInterface::trafficInterface = NULL;
int TrafficInterface::tIType = -1;

TrafficInterface::TrafficInterface() {
//	printf( "Constructor of TrafficInterface!\n" );

	this->downloadDiff = 0.0;
	this->uploadDiff = 0.0;
	this->oldDownload = 0.0;
	this->oldUpload = 0.0;
	this->interfaceChanged = false;

	this->interfaceName = "";
}

TrafficInterface::~TrafficInterface() {
//	printf( "Destructor of TrafficInterface!\n" );
}

TrafficInterface *TrafficInterface::getTrafficInterface( int newtIType ) {
	//Check if there is no traffic interface so far
	if( trafficInterface == NULL ) {
		tIType = newtIType;
		trafficInterface = TrafficInterface::spawnTrafficInterface( tIType );
	}
	//If one exists, check if type changed
	else if( tIType != newtIType ) {
		delete trafficInterface;
		tIType = newtIType;
		trafficInterface = TrafficInterface::spawnTrafficInterface( tIType );
	}

	return trafficInterface;
}

TrafficInterface *TrafficInterface::spawnTrafficInterface( int interfaceType ) {
	TrafficInterface *tI = NULL;

	switch( interfaceType ) {
		case 1:
			tI = new SYSTrafficInterface();
			break;
		//case 2:
			//tI = new SSHTrafficInterface();
		//	break;
		default:
			tI = new TrafficInterface();
			break;
	}

	return tI;
}

//Read traffic differences for interface interfaceName
//Error codes:
// < 1 - no error
// 1 - /proc/net/dev couldn't be read
// 2 - Interface doesn't exist
int TrafficInterface::readTrafficDiff( char *currInterfaceName ) {
	QString data;					//Data buffer for file content
	QFile procInfo( "/proc/net/dev" );	//Info about trafficUpload

	//Set interface name
	this->interfaceName = currInterfaceName;

	//Check if we have a different interface now (if yes reset values)
	if( QString::compare( currInterfaceName, this->lastInterfaceName ) != 0 ) {
		//printf( "Changed: %s / %s\n" , currInterfaceName, this->lastInterfaceName);
		this->downloadDiff = 0.0;
		this->uploadDiff = 0.0;
		this->oldDownload = 0.0;
		this->oldUpload = 0.0;
		this->lastInterfaceName = (char *) malloc( sizeof(char) * ( strlen(currInterfaceName) + 1 ) );
		//strcpy( this->lastInterfaceName, currInterfaceName );
                this->lastInterfaceName = currInterfaceName;
		//this->lastInterfaceName = QString( currInterfaceName ).toLatin1().data();
		this->interfaceChanged = true;
	}
	else {
		this->interfaceChanged = false;
	}

	procInfo.open( QIODevice::ReadOnly );
	//Check if file can be opened
	if( !procInfo.isOpen() ) {
		//...if not return error
		return 1;
	}

	QTextStream piIn(&procInfo);

	//Skip first 2 lines (only contain header data)
	piIn.readLine();
	piIn.readLine();
	//Now read correct interface
/*	for( int i = 0; i <= interfaceNum; i++ ) {
		procInfo->readLine( data, 4096 );
	}
	procInfo->close();*/

	//eth0: 5299187   15784    0    0    0     0          0         0  1139753   14261    0    0    0     0       0          0
	//Apply regular expression at our line
	QRegExp netInfo( "\\s*(\\S+):\\s*(\\d+)\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s+\\d+\\s*(\\d+)" );
	//Check if search was succesfull
	float trafficDownload;
	float trafficUpload;

    //Cycle through all available interfaces
//    while( procInfo->readLine( data, 4096 ) != -1 ) {
    while( !piIn.atEnd() ) {
    	data = piIn.readLine();
		if( netInfo.indexIn( data ) != -1 ) {
			//printf( "Found: %s / %s\n", currInterfaceName, netInfo.cap(1).toLatin1().constData() );
			if( QString::compare( currInterfaceName, netInfo.cap( 1 ) ) != 0 ) continue;

			//Get traffic values
			trafficDownload = netInfo.cap( 2 ).trimmed().toFloat();
			trafficUpload = netInfo.cap( 3 ).trimmed().toFloat();

			/*printf( "Line-Data: %s\n", data.toLatin1().constData() );
			printf( "Stats: %s / %s\n", netInfo.cap( 2 ).toLatin1().constData(), netInfo.cap( 3 ).toLatin1().constData() );
			printf( "Stats: %f / %f\n", trafficDownload, trafficUpload );*/

			//Check if first run
			if( this->oldDownload == 0.0 || this->oldUpload == 0.0 ) {
				this->oldDownload = trafficDownload;
				this->oldUpload = trafficUpload;
			}

			//Calculate differences
			this->downloadDiff = trafficDownload - this->oldDownload;
			this->uploadDiff = trafficUpload - this->oldUpload;

			/*printf( "Diff: %f / %f\n", this->downloadDiff, this->uploadDiff );
			printf( "Stats: %f / %f\n", trafficDownload, trafficUpload );
			printf( "Stats-Old: %f / %f\n", oldDownload, oldUpload );*/

			//Check if interface was up all time (else we might get a negative difference)
			if( this->downloadDiff < 0 || this->uploadDiff < 0 ) {
				this->downloadDiff = 0;
				this->uploadDiff = 0;
			}

			//Save new values
			this->oldDownload = trafficDownload;
			this->oldUpload = trafficUpload;

			//Close file
			procInfo.close();
			return 0;
		}
    }

	//Close file
	procInfo.close();
	return 2;
}

//Get a list of all available interfaces
QStringList TrafficInterface::getInterfaces() {
	//QStringList start
	QStringList interfaces;

    //Open net-dev file
    QString data;					//Data buffer for file content
    QFile procInfo( "/proc/net/dev" );	//Info about trafficUpload
    procInfo.open( QIODevice::ReadOnly );

    QTextStream piIn(&procInfo);

    //Skip first 2 lines
    piIn.readLine();
    piIn.readLine();
    //Read out available interfaces
    while( !piIn.atEnd() ) {
    	data = piIn.readLine();

       	QRegExp interfaceInfo( "^\\s*(\\S+):\\s*\\d+.*" );
		if( interfaceInfo.indexIn( data ) != -1 ) {
			interfaces << interfaceInfo.cap( 1 );
		}
    }
    //Close net-dev file
    procInfo.close();

    //Return interface list
    return interfaces;
}

//Get download traffic difference for last interface
float TrafficInterface::getDownloadDiff() {
	return this->downloadDiff;
}

//Get upload traffic difference for last interface
float TrafficInterface::getUploadDiff() {
	return this->uploadDiff;
}

//Get interface name (set by last call of readTrafficDiff)
QString TrafficInterface::getInterfaceName() {
	return this->interfaceName;
}

//Returns true if the interface change in the last readTrafficDiff call
bool TrafficInterface::getInterfaceChanged() {
	return this->interfaceChanged;
}

//Update Debug-Settings
void TrafficInterface::setDebug( bool bDebug ) {
    this->bDebug = bDebug;
}
