/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef TRAFFICINTERFACE
#define TRAFFICINTERFACE

#include <stdlib.h>

#include <QtCore/QString>
#include <QtCore/QFile>
#include <QtCore/QRegExp>
#include <QtCore/QTextStream>

class TrafficInterface {
	public:
		TrafficInterface();
		virtual ~TrafficInterface();

		static TrafficInterface *getTrafficInterface( int );    //Fills the class-pointer with the correct interface and returns it
		static TrafficInterface *spawnTrafficInterface( int );  //Spawns a TrafficInterface of correct type and returns it

		virtual int readTrafficDiff( char * );	//Read traffic differences for interface interfaceName
		virtual QStringList getInterfaces();	//Get a list of all available interfaces
		virtual QString getInterfaceName();		//Get interface name (set by last call of readTrafficDiff)

		float getDownloadDiff();			//Get download traffic difference for last interface
		float getUploadDiff();				//Get upload traffic difference for last interface

		bool getInterfaceChanged();			//Returns true if the interface change in the last readTrafficDiff call

		virtual void setDebug( bool );		//Update Debug-Settings

	protected:
		float downloadDiff, uploadDiff;
		float oldDownload, oldUpload;
		QString lastInterfaceName;
		bool interfaceChanged;
		QString interfaceName;
                bool bDebug;

		static TrafficInterface *trafficInterface;
		static int tIType;
};

#include "../lib/SYSTrafficInterface.h"
//#include "lib/SSHTrafficInterface.h"

#endif

