/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "TrafficWindow.h"
#include "TrafficWindow.moc"

//Compares two values and returns the bigger one
float maxVal(float val1, float val2) {
    if (val1 > val2) {
        return val1;
    } else {
        return val2;
    }
}

TrafficWindow::TrafficWindow(QWidget * parent, const char * name) : KMainWindow(parent, 0) {
    //Get TrafficInterface
    this->trafficInterface = TrafficInterface::getTrafficInterface(KTASettings::interfacesSource());

    //Check if interface still exists
    if (this->trafficInterface->readTrafficDiff(KTASettings::interface().toAscii().data()) == 2) {
        //Set back to first available interface
        KTASettings::setInterface(this->trafficInterface->getInterfaces().first());
        this->trafficInterface->readTrafficDiff((char *) KTASettings::interface().toAscii().data());
    }

    //Create SysTray icon
    this->sysTray = new KTASysTray(this, "KTASysTray");
    //this->sysTray->setInterfaceName(this->trafficInterface->getInterfaceName());
    this->sysTray->show();
    //Connect quit button
    QObject::connect(this->sysTray, SIGNAL(quitSelected()), KApplication::kApplication(), SLOT(quit()));
    //Connect the interface update
    QObject::connect(this->sysTray, SIGNAL(interfaceChanged(QString)), this, SLOT(changeInterface(QString)));

    //Create settings dialog
    KConfigDialog *dialog = new KConfigDialog(this, "KTASettings", KTASettings::self());
    //dialog->setFixedSize( 440, 350 );
    dialog->resize(440, 350);
    QObject::connect(dialog, SIGNAL(settingsChanged(QString)), this, SLOT(updateSettings(QString)));
    
    //Create window entry
    WindowWidget *windowWidget = new WindowWidget(dialog);
    dialog->addPage(windowWidget, i18n("Window"), "preferences-system-windows");
    //Create color entry
    ColorWidget *colorWidget = new ColorWidget(dialog);
    dialog->addPage(colorWidget, i18n("Colors"), "preferences-desktop-color");
    //Create interface entry
    InterfaceWidget *interfaceWidget = new InterfaceWidget(dialog);
    interfaceWidget->loadInterfaces(KTASettings::interfacesSource());
    dialog->addPage(interfaceWidget, i18n("Interface"), "hwinfo");
    //Create font entry
    FontWidget *fontWidget = new FontWidget(dialog);
    dialog->addPage(fontWidget, i18n("Fonts"), "preferences-desktop-font");
    /*	//Create SSH entry
            Ui::SSHWidget swui;
            QDialog *sshWidget = new QDialog();
            swui.setupUi( sshWidget );
            dialog->addPage( sshWidget, "SSH", "konsole" );*/
    //Create statistics entry
    StatisticsWidget *statisticsWidget = new StatisticsWidget(dialog);
    dialog->addPage(statisticsWidget, i18n("Statistics"), "kchart");

    //Create correct stats skeleton
    /*delete KTAStats::self();
    KTAStats::self(this->trafficInterface->getInterfaceName().prepend("ktastats"));
    //Create stats dialog
    stats = new KConfigDialog(this, "KTAStats", KTAStats::self());
    //stats->setFixedSize( 450, 430 );
    stats->resize(450, 430);
    stats->setCaption(i18n("Statistics") + "(" + this->trafficInterface->getInterfaceName() + ")");
    //Create traffic stats entry
    dailyStatsWidget = new DailyStatsWidget(stats);
    stats->addPage(dailyStatsWidget, i18n("Daily"), "accessories-calculator");
    //Create monthly traffic stats entry
    monthlyStatsWidget = new MonthlyStatsWidget();
    stats->addPage(monthlyStatsWidget, i18n("Monthly"), "accessories-calculator");
    //Update traffic stats in real-time
    QObject::connect(this, SIGNAL(updateStatsDialog(int)), dailyStatsWidget, SLOT(updateStatistics(int)));
    QObject::connect(this, SIGNAL(updateStatsDialog(int)), monthlyStatsWidget, SLOT(updateStatistics(int)));*/

    //Create display for realtime traffic graph
    this->graphDisplay = new GraphDisplay(this);
    this->graphDisplay->setAutoFillBackground(true);
    this->graphDisplay->show();
    //Create buffer-pixmap
    //this->paintDevice = new QPixmap( KTASettings::sizeX(), KTASettings::sizeY() );
    this->paintDevice = this->graphDisplay;

    //Create status bar
    this->statusBar = new QStatusBar(this);
    this->statusBar->setGeometry(0, this->height() - 14, this->width(), 14);
    this->statusBar->setSizeGripEnabled(false);
    this->statusBar->show();

    //Create menu bar
    QMenu *settingsMenu = new QMenu(i18n("Settings"), this);
    QAction *menuBarAction = settingsMenu->addAction(i18n("Show Menu-Bar"), this->menuBar(), SLOT(setVisible(bool)));
    menuBarAction->setShortcut(QKeySequence::fromString("Ctrl+M"));
    menuBarAction->setShortcutContext(Qt::ApplicationShortcut);
    menuBarAction->setCheckable(true);
    menuBarAction->setChecked(KTASettings::showToolBar());
    QObject::connect(menuBarAction, SIGNAL(triggered()), this, SLOT(menuBarHidden()));
    this->addAction(menuBarAction);

    settingsMenu->addAction(i18n("Setup KTrafficAnalyzer..."), dialog, SLOT(show()));
    //this->menuBar()->addAction( i18n( "Settings" ), dialog, SLOT( show() ) );
    this->menuBar()->addMenu(settingsMenu);
    //this->statsEntryId = this->menuBar()->addAction(i18n("Stats"), stats, SLOT(show()));
    this->exitEntryAction = this->menuBar()->addAction(i18n("Exit"), KApplication::kApplication(), SLOT(quit()));
    this->menuBar()->addMenu(this->helpMenu());
    this->statsEntryAction = NULL;

    //Create data points
    this->memoryUpload = new ringMemory(KTASettings::monitorTime());
    this->memoryDownload = new ringMemory(KTASettings::monitorTime());

    //Add context menu entries at the SysTray
    this->sysTray->contextMenu()->addAction(i18n("Settings"), dialog, SLOT(show()));
    this->sysTray->contextMenu()->addAction(i18n("Stats"), stats, SLOT(show()));

    //Restore size?
    if (KTASettings::rememberSize()) {
        this->resize(KTASettings::sizeX(), KTASettings::sizeY());
    } else {
        this->resize(250, 100);
    }
    //Restore position?
    if (KTASettings::rememberPos()) {
        this->move(KTASettings::posX(), KTASettings::posY());
    }

    // Setup window icon
    this->setWindowIcon(KIconLoader::global()->loadIcon("network-workgroup", KIconLoader::Toolbar));

    //Create timer
    this->timer = new QTimer();
    this->timerTicks = 0;
    //Connect timer signal to our own slot
    QObject::connect(this->timer, SIGNAL(timeout()), this, SLOT(timerTimeout()));

    //Apply all settings
    this->updateSettings();

    //timerTimeout();
    // Initialize all interface related settings
    this->interfaceChanged();

    //Start timer
    //this->timer->start( KTASettings::refreshTime() * 1000 );
    this->timer->start(1000);
}

//Destructor
TrafficWindow::~TrafficWindow() {
}

//Called every second
void TrafficWindow::timerTimeout() {
    // Count our timer ticks
    timerTicks++;

    //Update interface data (if refreshTime is over)
    if ((timerTicks % KTASettings::refreshTime()) == 0 && trafficInterface->readTrafficDiff((char *) KTASettings::interface().toAscii().data()) == 2) {
        //Set back to first available interface
        KTASettings::setInterface(this->trafficInterface->getInterfaces().first());
        //Init interface
        //this->trafficInterface->readTrafficDiff( (char *) KTASettings::interface().ascii() );
        this->updateSettings();

        KMessageBox::error(NULL, i18n("The previously selected interface went down. Switching back to first available one. Please review your settings!"), i18n("Interface Error!"));
    }

    //Get traffic differences
    float trafficDownloadDiff = trafficInterface->getDownloadDiff() / (float) KTASettings::refreshTime();
    float trafficUploadDiff = trafficInterface->getUploadDiff() / (float) KTASettings::refreshTime();
    //Add new elements with difference
    memoryDownload->newElement(trafficDownloadDiff);
    memoryUpload->newElement(trafficUploadDiff);

    //printf( "%f / %f\n", trafficDownloadDiff, trafficUploadDiff );

    //Show upload / download at status bar
    KLocale *locale = KGlobal::locale();
    QString statusMessage(i18n("Down") + ": ");
    statusMessage.append(locale->formatByteSize(trafficDownloadDiff).append("/s"));
    statusMessage.append(" " + i18n("Up") + ": ");
    statusMessage.append(locale->formatByteSize(trafficUploadDiff).append("/s"));
    this->statusBar->showMessage(statusMessage);
    //Show upload / download at tooltip
    this->sysTray->updateTraffic(locale->formatByteSize(trafficDownloadDiff).append("/s"), locale->formatByteSize(trafficUploadDiff).append("/s"));

    //Update traffic stats
    QStringList dates(KTAStats::dates());
    QStringList uploads(KTAStats::uploads());
    QStringList downloads(KTAStats::downloads());
    QString dateString(QDate::currentDate().toString("dd.MM.yyyy"));
    int dateIndex = dates.indexOf(dateString);
    //Date already exists
    if (dateIndex != -1) {
        downloads[dateIndex] = QString::number(downloads[dateIndex].toFloat() + trafficDownloadDiff);
        uploads[dateIndex] = QString::number(uploads[dateIndex].toFloat() + trafficUploadDiff);
    }//Date not added yet
    else {
        dates.append(dateString);
        downloads.append(QString::number(trafficDownloadDiff));
        uploads.append(QString::number(trafficUploadDiff));
        dateIndex = 0;

    }
    KTAStats::setDates(dates);
    KTAStats::setUploads(uploads);
    KTAStats::setDownloads(downloads);
    if (KTASettings::alwaysWriteStats()) KTAStats::self()->writeConfig();

    //Update dialog (if traffic was changed)
    if (trafficDownloadDiff != 0.0 || trafficUploadDiff != 0.0) {
        emit updateStatsDialog(dateIndex);
    }

    //Finally draw the graph
    //this->drawGraph();
    this->graphDisplay->repaint();
    //this->repaint();
}

//Update all settings (called by settings dialog)
void TrafficWindow::updateSettings(QString sender) {
    Qt::WindowFlags myFlags = Qt::Widget;

    //printf( "updateSettings: %s\n", dialogName.data()->toAscii() );

    //Setup Fonts
    this->statusBar->setFont(KTASettings::statusBarFont());
    this->menuBar()->setFont(KTASettings::menuBarFont());

    //Resize memory rings
    this->memoryUpload->resize(KTASettings::monitorTime());
    this->memoryDownload->resize(KTASettings::monitorTime());

    //Check if interfaces-Source has changed
    if (this->trafficInterface != TrafficInterface::getTrafficInterface(KTASettings::interfacesSource())) {
        //Get new interface handler and initialize it
        this->trafficInterface = TrafficInterface::getTrafficInterface(KTASettings::interfacesSource());
        this->trafficInterface->readTrafficDiff((char *) KTASettings::interface().toAscii().data());
    }

    // Update interface menu
    this->sysTray->updateInterfaces(this->trafficInterface->getInterfaces(), KTASettings::interface());

    //Check if interface changed
    if (this->trafficInterface->getInterfaceName().compare(KTASettings::interface()) != 0) {
        this->interfaceChanged();
        /*delete KTAStats::self();
        KTAStats::self(this->trafficInterface->getInterfaceName().prepend("ktastats"));

        //Create new stats dialog
        delete stats;
        stats = new KConfigDialog(this, "KTAStats", KTAStats::self());
        stats->setFixedSize(450, 430);
        stats->setCaption(i18n("Statistics") + " (" + this->trafficInterface->getInterfaceName() + ")");
        //Create traffic stats entry
        dailyStatsWidget = new DailyStatsWidget(stats);
        stats->addPage(dailyStatsWidget, i18n("Daily"), "nfs_unmount");
        //Create monthly traffic stats entry
        monthlyStatsWidget = new MonthlyStatsWidget();
        stats->addPage(monthlyStatsWidget, i18n("Monthly"), "nfs_unmount");
        //Update traffic stats in real-time
        QObject::connect(this, SIGNAL(updateStatsDialog(int)), dailyStatsWidget, SLOT(updateStats(int)));
        //Update menu-bar
        QAction *tmpAction = this->menuBar()->addAction(i18n("Stats"), stats, SLOT(show()));
        this->menuBar()->insertAction(this->statsEntryId, tmpAction);
        this->menuBar()->removeAction(this->statsEntryId);
        this->statsEntryId = tmpAction;
        //this->statsEntryId = this->menuBar()->addAction("Stats", stats, SLOT(show()));

        //Reset memory
        this->memoryUpload->reset();
        this->memoryDownload->reset();

        //Read first data-point
        this->trafficInterface->readTrafficDiff((char *) KTASettings::interface().toAscii().data());

        //Update toolTip
        this->sysTray->setInterfaceName(this->trafficInterface->getInterfaceName());*/
    }

    //Update Debug-Settings (only relevant for SSH-Connection atm)
    this->trafficInterface->setDebug(KTASettings::sSHDebug());

    //Set new timer-interval
    //this->timer->setInterval(KTASettings::refreshTime() * 1000);

    //Set background color
    QPalette windowPalette = this->graphDisplay->palette();
    windowPalette.setColor(QPalette::Window, KTASettings::backgroundColor());
    this->graphDisplay->setPalette(windowPalette);
    //	this->graphDisplay->setPaletteBackgroundColor( KTASettings::backgroundColor() );
    //Check if status bar should also use BG color
    if (KTASettings::sbUsesBGColor()) {
        //this->statusBar->setPaletteBackgroundColor( KTASettings::backgroundColor() );
        this->statusBar->setAutoFillBackground(true);
        QPalette sbPalette = this->statusBar->palette();
        sbPalette.setColor(QPalette::Window, KTASettings::backgroundColor());
        this->statusBar->setPalette(sbPalette);
    } else {
        this->statusBar->setAutoFillBackground(false);
        //this->statusBar->unsetPalette();
    }

    //Setup window flags
    if (KTASettings::noBorder()) {
        myFlags = myFlags | Qt::FramelessWindowHint;
    } else {
        myFlags = myFlags | Qt::Window;
    }
    //	if( this->testWFlags( myFlags ) != myFlags ) {
    //		this->reparent( 0, myFlags, this->pos(), true );
    //	}
    // Apply flags and issue show again (required because on setWindowFlags the window gets hidden)
    this->setWindowFlags(myFlags);
    this->show();

    //Check if window should stay on top
    if (KTASettings::alwaysOnTop()) {
        KWindowSystem::setState(this->winId(), NET::StaysOnTop);
    } else {
        KWindowSystem::clearState(this->winId(), NET::StaysOnTop);
    }

    // Setup window title & icon
    //this->setWindowTitle("KTrafficAnalyzer (" + this->trafficInterface->getInterfaceName() + ")");
    //this->setWindowIcon(KIconLoader::global()->loadIcon("network-workgroup", KIconLoader::Toolbar));

    //Change caption of statistics dialog
    //this->stats->setCaption(i18n("Statistics") + " (" + this->trafficInterface->getInterfaceName() + ")");

    //Check if toolbar should be shown or not
    if (KTASettings::showToolBar()) {
        this->menuBar()->show();
    } else {
        this->menuBar()->hide();
    }
    //Call resize so that a resizeEvent is emitted and the display gets updated
    this->resizeEvent(NULL);

    //Check if we should skip taskbar entry
    if (KTASettings::skipTaskBar()) {
        KWindowSystem::setState(this->winId(), NET::SkipTaskbar);
    } else {
        KWindowSystem::clearState(this->winId(), NET::SkipTaskbar);
    }

    //Check if we should show on all desktops
    if (KTASettings::showOnAllDesktops()) {
        KWindowSystem::setOnAllDesktops(this->winId(), true);
    } else {
        KWindowSystem::setOnAllDesktops(this->winId(), false);
    }

    // Apply window opacity
    this->setWindowOpacity(KTASettings::opacityLevel() / 100.0);

    // Setup autostart
    KAutostart autostart;
    autostart.setAutostarts(KTASettings::autostart());
    //autostart.setCommand( "ktrafficanalyzer" );
    autostart.setStartPhase(KAutostart::Applications);
}

//Converts a value to a string (containing SI units)
/*QString TrafficWindow::getDisplayString( float value, bool bNoExt ) {
        float correctValue;
        QString SIUnit( " " );

        //Check for MBytes
        if( ( value / 1024000.0 ) > 1.0) {
                SIUnit.append( "M" );
                correctValue = value / 1024000.0;
        }
        //Check for KBytes
        else if( ( value / 1024.0 ) > 1.0 ) {
                SIUnit.append( "K" );
                correctValue = value / 1024.0;
        }
        //Normal Bytes
        else {
                correctValue = value;
        }

        //Create return QString
        QString displayString( SIUnit );
        displayString.append( "B" );
        if( !bNoExt ) {
                displayString.append( "/s" );
        }
        displayString.prepend( QString::number( correctValue, 'f', 1 ) );

        return displayString;
}*/

//Draw the traffic-graph
void TrafficWindow::drawGraph() {
    //Draw grid lines
    drawGrid();
    //Draw both memory buffers
    drawRingMemory(memoryDownload, maxVal(memoryDownload->maxValue(), memoryUpload->maxValue()), KTASettings::downloadColor());
    drawRingMemory(memoryUpload, maxVal(memoryDownload->maxValue(), memoryUpload->maxValue()), KTASettings::uploadColor());
}

//Draws a grid onto the display
void TrafficWindow::drawGrid() {
    //Get size of graph window
    int sizeX = this->paintDevice->width();
    int sizeY = this->paintDevice->height();

    QPainter windowPainter(this->paintDevice);
    windowPainter.setPen(KTASettings::gridColor());

    int numlines = (int) ((float) sizeY / 10.0);
    for (int i = 1; i <= numlines; i++) {
        windowPainter.drawLine(0, i * 10, sizeX, i * 10);
    }

    windowPainter.end();
}

//Draws a whole ringMemory
int TrafficWindow::drawRingMemory(ringMemory *dataPoints, float maxValue, QColor color) {
    //start painting
    QPainter windowPainter(this->paintDevice);
    windowPainter.setFont(KTASettings::statusBarFont());
    windowPainter.setPen(KApplication::palette().color(QPalette::Foreground));
    windowPainter.drawText(QRect(1, 0, 100, 100), Qt::AlignLeft | Qt::AlignTop, KGlobal::locale()->formatByteSize(maxValue));
    windowPainter.setPen(color);

    // Initialize variables
    dataPoint *currValue = dataPoints->nextValue();
    int currPos = 0;
    int startX = 0;
    int startY = 0;
    int endX = 0;
    int endY = 0;
    int dpCount = dataPoints->count();

    // Draw all points
    while (currValue != NULL) {
        endX = (int) (((float) currPos / (float) (dpCount - 1)) * (float) this->paintDevice->width());
        endY = this->paintDevice->height() - (int) (currValue->getValue() / maxValue * (float) this->paintDevice->height());

        if (startX != 0 || startY != 0) windowPainter.drawLine(startX, startY, endX, endY);

        startX = endX;
        startY = endY;

        currValue = dataPoints->nextValue();
        currPos++;
    }

    windowPainter.end();

    return (int) dataPoints->count();
}

// Called when the menubar gets hidden
void TrafficWindow::menuBarHidden() {
    this->resizeEvent(NULL);
}

// Called to change / update the interface directly
void TrafficWindow::changeInterface(QString interfaceName) {
    //printf( "'%s'\n", interfaceName.toAscii().data() );
    KTASettings::setInterface(interfaceName);
    this->interfaceChanged();

    //    this->updateSettings();
}

//Called whenever our main window is resize
void TrafficWindow::resizeEvent(QResizeEvent *resizeEvent) {
    int menuHeight = this->menuBar()->heightForWidth(this->width());

    if (!KTASettings::showToolBar() || !this->menuBar()->isVisible()) {
        menuHeight = 0;
    }

    this->statusBar->setGeometry(0, this->height() - this->statusBar->height(), this->width(), this->statusBar->height());
    this->graphDisplay->setGeometry(0, menuHeight, this->width(), this->height() - this->statusBar->height() - menuHeight);
    //this->paintDevice->scaled( this->width(), this->height() - this->statusBar->height() - menuHeight );

    // If we resize the window, and want to remember the size, we have to write it to disk
    if (KTASettings::rememberSize()) {
        KTASettings::setSizeX(this->width());
        KTASettings::setSizeY(this->height());
        KTASettings::self()->writeConfig();
    }
}

// Called when the window is moved
void TrafficWindow::moveEvent(QMoveEvent *moveEvent) {
    // If the window gets moved, and rememberPos is on, write the current position to disk
    if (KTASettings::rememberPos()) {
        KTASettings::setPosX(this->x());
        KTASettings::setPosY(this->y());
        KTASettings::self()->writeConfig();
    }
}

//Called when widget gets hidden
void TrafficWindow::hideEvent(QHideEvent *hideEvent) {
    //printf( "Hide-Event! (%d / %d)\n", this->width(), this->height() );
    if (KTASettings::minimizeToTray() && !this->isMaximized()) {
        //printf( "Minimized!\n" );
        this->hide();
    }
}

void TrafficWindow::interfaceChanged() {
    // Remove old stats handler and create a new one with for the changed interface
    delete KTAStats::self();
    KTAStats::self(this->trafficInterface->getInterfaceName().prepend("ktastats"));

    //Create new stats dialog
    if( stats != NULL ) delete stats;
    stats = new KConfigDialog(this, "KTAStats", KTAStats::self());
    stats->setFixedSize(450, 430);
    stats->setCaption(i18n("Statistics") + " (" + this->trafficInterface->getInterfaceName() + ")");
    //Create traffic stats entry
    dailyStatsWidget = new DailyStatsWidget(stats);
    stats->addPage(dailyStatsWidget, i18n("Daily"), "nfs_unmount");
    //Create monthly traffic stats entry
    monthlyStatsWidget = new MonthlyStatsWidget();
    stats->addPage(monthlyStatsWidget, i18n("Monthly"), "nfs_unmount");
    //Update traffic stats in real-time
    QObject::connect(this, SIGNAL(updateStatsDialog(int)), dailyStatsWidget, SLOT(updateStatistics(int)));
    QObject::connect(this, SIGNAL(updateStatsDialog(int)), monthlyStatsWidget, SLOT(updateStatistics(int)));

    //Update menu-bar
    this->menuBar()->removeAction(this->statsEntryAction);
    this->statsEntryAction = this->menuBar()->addAction(i18n("Stats"), stats, SLOT(show()));
    this->menuBar()->insertAction(this->exitEntryAction, this->statsEntryAction);

    //Reset memory
    this->memoryUpload->reset();
    this->memoryDownload->reset();

    //Read first data-point
    this->trafficInterface->readTrafficDiff((char *) KTASettings::interface().toAscii().data());

    //Update toolTip
    this->sysTray->setInterfaceName(this->trafficInterface->getInterfaceName());

    // Setup window title
    this->setWindowTitle("KTrafficAnalyzer (" + this->trafficInterface->getInterfaceName() + ")");

    //Change caption of statistics dialog
    this->stats->setCaption(i18n("Statistics") + " (" + this->trafficInterface->getInterfaceName() + ")");
}
