/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef TRAFFICWINDOW
#define TRAFFICWINDOW

const char versionString[] = "0.5.4";

#include <stdlib.h>

#include <QtGui/QPainter>
#include <QtCore/QTimer>
#include <QtCore/QFile>
#include <QtCore/QRegExp>
#include <QtGui/QMenuBar>
#include <QtGui/QStatusBar>
#include <QtGui/QLabel>
#include <QtGui/QFont>
#include <QtGui/QColor>
#include <QtGui/QPixmap>
#include <QtGui/QMenu>
#include <QtGui/QResizeEvent>

#include <kapplication.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kaboutapplicationdialog.h>
//#include <Qt3Support/q3popupmenu.h>
#include <kiconloader.h>
#include <kglobalsettings.h>
#include <kmainwindow.h>
#include <kwindowsystem.h>
#include <kmenubar.h>
#include <kmenu.h>
#include <kautostart.h>

#include "ringMemory.h"
#include "KTASysTray.h"
#include "TrafficInterface.h"
#include "SYSTrafficInterface.h"
//#include "SSHTrafficInterface.h"

#include "KTASettings.h"

#include "../settings/ColorWidget.h"
#include "../settings/InterfaceWidget.h"
#include "../settings/FontWidget.h"
#include "../settings/WindowWidget.h"
//#include "../settings/SSHWidget.cpp"
#include "../settings/StatisticsWidget.h"

#include "../stats/KTAStats.h"
//#include "../stats/TrafficStatsWidget.h"
#include "../stats/DailyStatsWidget.h"
#include "../stats/MonthlyStatsWidget.h"

class TrafficWindow;

#include "GraphDisplay.h"

class TrafficWindow : public KMainWindow {
    Q_OBJECT

public:
    TrafficWindow(QWidget * parent = 0, const char * name = 0);
    ~TrafficWindow();

public slots:
    void timerTimeout();
    void updateSettings(QString = "");
    void drawGraph();                   //Draw the traffic-graph
    void menuBarHidden();               // Called when the menu-bar is hidden using the shortcut => triggers a resize event
    void changeInterface(QString);      // Called to change / update the interface directly

protected:
    //QString getDisplayString( float, bool = false );
    void drawGrid();                                    //Draw grid on traffic-graph
    int drawRingMemory(ringMemory *, float, QColor);    //Draw the curves onto the traffic-graph

    void resizeEvent(QResizeEvent *);
    void hideEvent(QHideEvent *);
    void moveEvent(QMoveEvent *);

signals:
    void updateStatsDialog(int);

private:
    void interfaceChanged();
    //void changeInterface();

    QTimer *timer;      //Timer for creating datapoints
    uint timerTicks;    // Counts the timer ticks

    ringMemory *memoryUpload;   //Memory-ring which holds upload data points
    ringMemory *memoryDownload; //Memory-ring which holds download data points

    TrafficInterface *trafficInterface; //Interface for monitoring the network traffic

    KTASysTray *sysTray;    //SysTray Icon for KTA

    GraphDisplay *graphDisplay; //Widget which is used to paint the graph
    //KMenuBar *menuBar;        //Menu bar
    QStatusBar *statusBar;      //Bar displaying current traffic ( up & down )

    //KAboutApplicationDialog *aboutDialog;		//About dialog
    KConfigDialog *stats;   //Stats dialog

    DailyStatsWidget *dailyStatsWidget;
    MonthlyStatsWidget *monthlyStatsWidget;
    QAction *statsEntryAction;
    QAction *exitEntryAction;

    QPaintDevice *paintDevice;
};

#endif

