/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

 #include "remoteRead.h"

//Constructor 
remoteRead::remoteRead() : KProcess() {
	this->bLoggedIn = false;
	this->bDebug = true;
}

remoteRead::~remoteRead() {
	if( this->bLoggedIn ) this->logout();
}

//Enable / Disable Debug
void remoteRead::setDebug( bool bNewDebug ) {
	this->bDebug = bNewDebug;
}

//Login in via SSH
void remoteRead::login( char *password ) {
	//Wait for the password request to appear
	char lineBuff[4096];

	//Wait for the password prompt
	this->readPtyUntil( "password: " );
	
	//Send password
	this->writePtyLine( password );
	
	//Wait for the command-prompt to appear
	this->readPtyUntil( ":~ # " );
	
	this->bLoggedIn = true;
}

//Logout from SSH
void remoteRead::logout() {
	this->writePtyLine( "exit" );
	this->bLoggedIn = false;
	this->readPtyUntil( "\r\r\n" );
	
}


/*//Login to remote host (using ssh)
void remoteRead::readTrafficLine( char *interfaceName, char *returnBuffer ) {
	//Check if we are logged in
	if( !this->bLoggedIn ) return;

	//Create command line
	QString command( "cat /proc/net/dev | grep " );
	command.append( interfaceName );
	
	//Send command to remote host
	this->writePtyLine( (char *) command.ascii(), strlen( command.ascii() ) );
	
	//Read return stuff
	char buffer[4096];
	this->readPtyUntil( ":~ # ", buffer );

	//Split up buffer into lines
	QStringList bufferLines = QStringList::split( "\n", buffer );
	//The seoncd line should be our data
	QString trafficInfo = bufferLines[1];
	
	printf( "Traffic-Info: %s\n", trafficInfo.ascii() );
	
	strcpy( returnBuffer, trafficInfo.ascii() );
}*/

//Read from Pty until the string "findString" is found
int remoteRead::readPtyUntil( char *findString, char *returnString ) {
	char smallBuff[] = "C";
	char buffer[4096];
	int readFd = this->pty()->masterFd();
	
	int i = 0;
	for( i = 0; i < sizeof( buffer ); i++ ) {
		read( readFd, smallBuff, 1 );
		buffer[i] = smallBuff[0];
		buffer[i+1] = '\0';
		
//		printf( "Part-Buffer: %s (%d)!\n", buffer, buffer[i] );
		
		QString bufferString( buffer );
		if( bufferString.find( findString, 0, false ) >= 0 ) break;
	}
	
	//Check if the line should be returned
	if( returnString != NULL ) strcpy( returnString, buffer );
	
	if( this->bDebug ) printf( "%s", buffer );
	
	return i;
}

//Read a line from the pty-device
int remoteRead::readPtyLine( char *buffer ) {	
	return this->readPtyUntil( "\n", buffer );
}

//Write a line to the pty-device
int remoteRead::writePtyLine( char *line ) {
	int writeFd = this->pty()->masterFd();
	
	if( this->bDebug ) printf( "%s\n", line );
	
	write( writeFd, line, strlen(line) );
	write( writeFd, "\n", 1 );
	
	return 0;
}
