/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "ringMemory.h"

ringMemory::ringMemory( int numElements ) {
	this->elementCount = numElements;

	this->pFirst = new bufferElement;
	this->pFirst->pData = new dataPoint( 0.0 );
	bufferElement *pCurr = this->pFirst;

	//Create a linked list
	for(int i = 1; i < numElements; i++ ) {
		bufferElement *newElement = new bufferElement;
		newElement->pData = new dataPoint( 0.0 );

                pCurr->pNext = newElement;
                newElement->pPrev = pCurr;
                pCurr = newElement;
	}

	//Finally connect the end and the start together
        pCurr->pNext = this->pFirst;
        // Reset internal arrow for loops
        this->pCurrent = NULL;
}

int ringMemory::newElement( float newValue ) {
    this->pFirst->pData->setValue( newValue );
    this->pFirst = this->pFirst->pNext;
    this->pCurrent = NULL;

    return 0;
}

int ringMemory::count() {
	return this->elementCount;
}

float ringMemory::maxValue() {
	bufferElement *temp = this->pFirst->pNext;
	float maxValue = this->pFirst->pData->getValue();

        while( temp != this->pFirst ) {
            if( temp->pData->getValue() > maxValue ) {
                maxValue = temp->pData->getValue();
            }

            temp = temp->pNext;
        }

	return maxValue;
}

// Returns NULL if one full loop is done
dataPoint *ringMemory::nextValue() {
    // Check if we reached the end
    if( this->pCurrent == this->pFirst ) {
        this->pCurrent = NULL;
        
        return NULL;
    }

    // Check if we have a fresh start
    if( this->pCurrent == NULL ) this->pCurrent = this->pFirst;

    dataPoint *currValue = this->pCurrent->pData;

    // Rotate next
    this->pCurrent = this->pCurrent->pNext;

    return currValue;
}

//Resizes the ring to a new size, if smaller it gets removed from the end
void ringMemory::resize( int newCount ) {
	if( newCount < this->elementCount ) {
		int sizeDiff = this->elementCount - newCount;

		//Delete number of elements
		bufferElement *temp = this->pFirst;
                bufferElement *pLast = this->pFirst->pPrev;
                //this->pFirst = temp->pPrev;
                
		for( int i = 1; i <= sizeDiff; i++ ) {
                    temp = temp->pNext;

                    delete temp->pPrev->pData;
                    delete temp->pPrev;
		}

		//Complete ring again
		temp->pPrev = pLast;
                pLast->pNext = temp;

                this->pFirst = temp;
	}
	else if( newCount > this->elementCount ) {
		int sizeDiff = newCount - this->elementCount;

		//bufferElement *temp = this->pFirst;
                bufferElement *pLast = this->pFirst->pPrev;

		for( int i = 0; i < sizeDiff; i++ ) {
			//Create new element
			bufferElement *newElement = new bufferElement;
			newElement->pData = new dataPoint( 0.0 );

                        newElement->pNext = this->pFirst;
                        this->pFirst->pPrev = newElement;
                        this->pFirst = newElement;
		}
		//Complete ring again
                pLast->pNext = this->pFirst;
                this->pFirst->pPrev = pLast;
	}

        // Reset internal pointers
	this->elementCount = newCount;
        this->pCurrent = NULL;
}

//Resets all values (sets them to 0) in the ring memory
void ringMemory::reset() {
    bufferElement *pTemp = this->pFirst->pNext;
    this->pFirst->pData->reset();

    while( pTemp != this->pFirst ) {
        pTemp->pData->reset();
        pTemp = pTemp->pNext;
    }
}
