/*
 * File:   DailyStatsWidget.cpp
 * Author: wkoller
 *
 * Created on 17. November 2009, 19:43
 */

#include "DailyStatsWidget.h"
#include "DailyStatsWidget.moc"

// TODO: Continue re-working the stats dialog

DailyStatsWidget::DailyStatsWidget(QWidget* parent) : QWidget(parent) {
    this->setupUi(this);

    // Setup horizontal header
    QStringList hHeaders;
    hHeaders << i18n("Uploads") << i18n("Downloads") << i18n("Totals");
    this->trafficTableWidget->setHorizontalHeaderLabels(hHeaders);

    // No stats read yet, will be done on first show
    this->bStatsRead = false;
}

void DailyStatsWidget::readStats() {
    // Load all stat entries
    QStringList uploads = KTAStats::uploads();
    QStringList downloads = KTAStats::downloads();
    QStringList dates = KTAStats::dates();

    // Basic variables required for adding the items
    QTableWidgetItem *nItem = NULL;
    float uploadVal, downloadVal = 0;
    KLocale *locale = KGlobal::locale();

    // Expand the table
    this->trafficTableWidget->setRowCount(uploads.size());

    // Finally cycle through all entries and add them
    for (int i = 0; i < uploads.size(); i++) {
        uploadVal = uploads[i].toFloat();
        downloadVal = downloads[i].toFloat();

        // Add entries for upload, download and totals
        nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(i, 0, nItem);
        nItem = new QTableWidgetItem(locale->formatByteSize(downloadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(i, 1, nItem);
        nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal + downloadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(i, 2, nItem);
    }

    // Setup vertical headers (which means the dates)
    QStringList vHeaders;
    for (int i = 0; i < dates.size(); i++) vHeaders << dates[i];
    this->trafficTableWidget->setVerticalHeaderLabels(vHeaders);

    // Initial stats reading done
    this->bStatsRead = true;
}

void DailyStatsWidget::showEvent( QShowEvent *showEvent ) {
    if( !this->bStatsRead ) this->readStats();
}

void DailyStatsWidget::updateStatistics(int dateIndex) {
    float uploadVal = KTAStats::uploads()[dateIndex].toFloat();
    float downloadVal = KTAStats::downloads()[dateIndex].toFloat();
    QString date = KTAStats::dates()[dateIndex];

    QTableWidgetItem *nItem = NULL;
    KLocale *locale = KGlobal::locale();

    if (this->trafficTableWidget->rowCount() <= dateIndex) {
        this->trafficTableWidget->setRowCount(dateIndex + 1);
        this->trafficTableWidget->setVerticalHeaderItem( dateIndex, new QTableWidgetItem( date ) );

        // Create the new item entries
        nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(dateIndex, 0, nItem);
        nItem = new QTableWidgetItem(locale->formatByteSize(downloadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(dateIndex, 1, nItem);
        nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal + downloadVal));
        nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        this->trafficTableWidget->setItem(dateIndex, 2, nItem);
    } else {
        this->trafficTableWidget->item(dateIndex, 0)->setText(locale->formatByteSize(uploadVal));
        this->trafficTableWidget->item(dateIndex, 1)->setText(locale->formatByteSize(downloadVal));
        this->trafficTableWidget->item(dateIndex, 2)->setText(locale->formatByteSize(uploadVal + downloadVal));
    }
}

/**
 * Called when the export Button gets clicked
 * Display a file selection dialog and export the content
 */
void DailyStatsWidget::on_exportButton_clicked() {
    // Get Filename to save to
    QString fileName = KFileDialog::getSaveFileName( KUrl(), "*.csv|CSV Files", this, i18n( "Select Export File" ) );

    // Check if user selected a file
    if( fileName.isEmpty() ) return;

    //Open file for writing
    QFile outFile(fileName);
    if( outFile.open(QIODevice::WriteOnly | QIODevice::Text) ) {
        //Create output stream
        QTextStream outStream(&outFile);

        //Write header
        for( int i = 0; i < this->trafficTableWidget->columnCount(); i++ ) {
            outStream << this->trafficTableWidget->horizontalHeaderItem( i )->text() << ";";
        }
        outStream << "\n";

        //Write data
        for (int i = 0; i < this->trafficTableWidget->rowCount(); i++) {
            // Write the date
            outStream << this->trafficTableWidget->verticalHeaderItem( i )->text() << ";";
            // Write all columns (generic, because we might want to add columns later on)
            for( int j = 0; j < this->trafficTableWidget->columnCount(); j++ ) {
                outStream << this->trafficTableWidget->item( i, j )->text() << ";";
            }
            outStream << "\n";
        }
        outFile.close();
    }
    // Something went wrong, display error
    else {
        KMessageBox::error(this, i18n( "Couldn't write to file " ) + ": " + outFile.errorString() + "!", i18n( "Export Error" ) ) ;
    }
}
