/*
 * File:   DailyStatsWidget.cpp
 * Author: wkoller
 *
 * Created on 17. November 2009, 19:43
 */

#include "MonthlyStatsWidget.h"

MonthlyStatsWidget::MonthlyStatsWidget(QWidget* parent) : DailyStatsWidget( parent ) {
}

void MonthlyStatsWidget::readStats() {
    // Load all stat entries
    QStringList uploads = KTAStats::uploads();
    QStringList downloads = KTAStats::downloads();
    QStringList dates = KTAStats::dates();

    // Basic variables required for adding the items
    QTableWidgetItem *nItem = NULL;
    float uploadVal = 0, downloadVal = 0;
    KLocale *locale = KGlobal::locale();

    // Expand the table
    //this->trafficTableWidget->setRowCount(uploads.size());

    QString newMonthString = QString::null;
    QString currMonthString = QDate::currentDate().toString( "MM.yyyy" );
    int currRowCount = 0;

    // Cycle through all entries and calculate month sums
    for (int i = 0; i < uploads.size(); i++) {
        newMonthString = dates[i].right( 7 );

        // Check if this is the first round
        if( i == 0 ) currMonthString = newMonthString;

        if( newMonthString != currMonthString ) {
            //int currRowCount = this->trafficTableWidget->rowCount();
            this->trafficTableWidget->setRowCount( currRowCount + 1 );
            this->trafficTableWidget->setVerticalHeaderItem( currRowCount, new QTableWidgetItem( currMonthString ) );

            // Add entries for upload, download and totals
            nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal));
            nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
            this->trafficTableWidget->setItem(currRowCount, 0, nItem);
            nItem = new QTableWidgetItem(locale->formatByteSize(downloadVal));
            nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
            this->trafficTableWidget->setItem(currRowCount, 1, nItem);
            nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal + downloadVal));
            nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
            this->trafficTableWidget->setItem(currRowCount, 2, nItem);

            // Reset upload / download counters
            uploadVal = 0;
            downloadVal = 0;
            // Increate row counter
            currRowCount++;
        }

        // Update month string
        currMonthString = newMonthString;
        // Update traffic values
        uploadVal += uploads[i].toFloat();
        downloadVal += downloads[i].toFloat();
    }

    // Add the final row
    this->trafficTableWidget->setRowCount(currRowCount + 1);
    this->trafficTableWidget->setVerticalHeaderItem(currRowCount, new QTableWidgetItem(currMonthString));

    // Add entries for upload, download and totals
    nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal));
    nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    this->trafficTableWidget->setItem(currRowCount, 0, nItem);
    nItem = new QTableWidgetItem(locale->formatByteSize(downloadVal));
    nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    this->trafficTableWidget->setItem(currRowCount, 1, nItem);
    nItem = new QTableWidgetItem(locale->formatByteSize(uploadVal + downloadVal));
    nItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    this->trafficTableWidget->setItem(currRowCount, 2, nItem);

    // Initial stats reading done
    this->bStatsRead = true;
}

void MonthlyStatsWidget::updateStatistics(int dateIndex) {
    // Not the best way to implement it, however until we have a better stats saving format we will use it that way
    this->trafficTableWidget->clearContents();
    this->trafficTableWidget->setRowCount( 0 );
    // Re-Read all stats
    this->readStats();
}
